@@ -4,6 +4,7 @@ import (
 	"crypto/sha256"
 	"fmt"
 	"io"
+	"io/ioutil"
 	"os"
 	"os/exec"
 	"path/filepath"
@@ -13,8 +14,8 @@ import (
 
 type RC struct {
 	path      string
-	mtime     int64
 	allowPath string
+	times     FileTimes
 }
 
 func FindRC(wd string, allowDir string) *RC {
@@ -27,28 +28,24 @@ func FindRC(wd string, allowDir string) *RC {
 }
 
 func RCFromPath(path string, allowDir string) *RC {
-	mtime, err := fileMtime(path)
-	if err != nil {
-		return nil
-	}
-
 	hash, err := fileHash(path)
 	if err != nil {
 		return nil
 	}
 
 	allowPath := filepath.Join(allowDir, hash)
-	allowMtime, _ := fileMtime(allowPath)
 
-	if allowMtime > mtime {
-		mtime = allowMtime
-	}
+	times := NewFileTimes()
+	times.Update(path)
+	times.Update(allowPath)
 
-	return &RC{path, mtime, allowPath}
+	return &RC{path, allowPath, times}
 }
 
-func RCFromEnv(path string, mtime int64) *RC {
-	return &RC{path, mtime, ""}
+func RCFromEnv(path, marshalled_times string) *RC {
+	times := NewFileTimes()
+	times.Unmarshal(marshalled_times)
+	return &RC{path, "", times}
 }
 
 func (self *RC) Allow() (err error) {
@@ -58,10 +55,10 @@ func (self *RC) Allow() (err error) {
 	if err = os.MkdirAll(filepath.Dir(self.allowPath), 0755); err != nil {
 		return
 	}
-	if err = touch(self.allowPath); err != nil {
+	if err = allow(self.path, self.allowPath); err != nil {
 		return
 	}
-	self.mtime, err = fileMtime(self.allowPath)
+	self.times.Update(self.allowPath)
 	return
 }
 
@@ -98,6 +95,8 @@ const NOT_ALLOWED = "%s is blocked. Run `direnv allow` to approve its content."
 func (self *RC) Load(config *Config, env Env) (newEnv Env, err error) {
 	wd := config.WorkDir
 	direnv := config.SelfPath
+	shellEnv := env.Copy()
+	shellEnv[DIRENV_WATCHES] = self.times.Marshal()
 
 	if !self.Allowed() {
 		return nil, fmt.Errorf(NOT_ALLOWED, self.RelTo(wd))
@@ -109,7 +108,7 @@ func (self *RC) Load(config *Config, env Env) (newEnv Env, err error) {
 
 	cmd.Stdin = os.Stdin
 	cmd.Stderr = os.Stderr
-	cmd.Env = env.ToGoEnv()
+	cmd.Env = shellEnv.ToGoEnv()
 	cmd.Dir = wd
 
 	out, err := cmd.Output()
@@ -122,14 +121,16 @@ func (self *RC) Load(config *Config, env Env) (newEnv Env, err error) {
 		return
 	}
 
-	// Save state
-	newEnv[DIRENV_DIR] = "-" + filepath.Dir(self.path)
-	newEnv[DIRENV_MTIME] = fmt.Sprintf("%d", self.mtime)
-	newEnv[DIRENV_DIFF] = env.Diff(newEnv).Serialize()
+	self.RecordState(env, newEnv)
 
 	return
 }
 
+func (self *RC) RecordState(env Env, newEnv Env) {
+	newEnv[DIRENV_DIR] = "-" + filepath.Dir(self.path)
+	newEnv[DIRENV_DIFF] = env.Diff(newEnv).Serialize()
+}
+
 /// Utils
 
 func rootDir(path string) string {
@@ -174,15 +175,6 @@ func fileExists(path string) bool {
 	return err == nil
 }
 
-func fileMtime(path string) (int64, error) {
-	fileInfo, err := os.Stat(path)
-	if err != nil {
-		return 0, err
-	}
-
-	return fileInfo.ModTime().Unix(), nil
-}
-
 func fileHash(path string) (hash string, err error) {
 	if path, err = filepath.Abs(path); err != nil {
 		return
@@ -205,16 +197,14 @@ func fileHash(path string) (hash string, err error) {
 // Creates a file
 
 func touch(path string) (err error) {
-	file, err := os.OpenFile(path, os.O_CREATE, 0644)
-	if err != nil {
-		return
-	}
-	file.Close()
-
 	t := time.Now()
 	return os.Chtimes(path, t, t)
 }
 
+func allow(path string, allowPath string) (err error) {
+	return ioutil.WriteFile(allowPath, []byte(path+"\n"), 0644)
+}
+
 func findUp(searchDir string, fileName string) (path string) {
 	for _, dir := range eachDir(searchDir) {
 		path = filepath.Join(dir, fileName)
